package org.jboss.as.test.integration.ws.authentication;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Service;
import javax.xml.ws.WebServiceException;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.arquillian.test.api.ArquillianResource;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

@RunWith(Arquillian.class)
public class EJBEndpointAuthenticationTestCase {

	@ArquillianResource
	private URL baseUrl;

	private EJBEndpointIface proxy;

	public static final String WEBAPP_SRC = "src/main/webapp";

	QName serviceName = new QName("http://jbossws.org/authentication",
			"EJB3AuthService");

	@Deployment(testable = false)
	public static Archive<?> deployment() {
		WebArchive archive = ShrinkWrap
				.create(WebArchive.class, "jaxws-authentication-ejb.war")
				.addAsWebInfResource(
						new File(WEBAPP_SRC, "WEB-INF/jboss-webservices.xml"))
				.addAsWebInfResource(
						new File(WEBAPP_SRC, "WEB-INF/jboss-ejb3.xml"))
				.addClasses(EJBEndpointIface.class, EJBEndpoint.class);
		archive.toString(true);
		return archive;
	}

	@Before
	public void setUp() throws MalformedURLException {
		URL wsdlURL = new URL(baseUrl,
				"/jaxws-authentication-ejb3/EJB3AuthService?wsdl");

		Service service = Service.create(wsdlURL, serviceName);
		proxy = service.getPort(EJBEndpointIface.class);
	}

	/* Tests for helloStranger method */

	@Test
	public void invokeHelloStrangerWithoutUsernameAndPassord() throws Exception {
		final String result = proxy.helloStranger("World");
		Assert.assertEquals("Hello World!", result);
	}

	@Test
	public void invokeHelloStrangerWithBadPassword() throws Exception {
		Map<String, Object> reqContext = ((BindingProvider) proxy)
				.getRequestContext();
		reqContext.put(BindingProvider.USERNAME_PROPERTY, "user");
		reqContext.put(BindingProvider.PASSWORD_PROPERTY, "badPassword");

		final String result = proxy.helloStranger("World");
		Assert.assertEquals("Hello World!", result);
	}

	@Test
	public void invokeHelloStrangerWithUnauthorizedUser() throws Exception {
		Map<String, Object> reqContext = ((BindingProvider) proxy)
				.getRequestContext();
		reqContext.put(BindingProvider.USERNAME_PROPERTY, "unauthorizedUser");
		reqContext.put(BindingProvider.PASSWORD_PROPERTY, "doesNotMatter");

		final String result = proxy.helloStranger("World");
		Assert.assertEquals("Hello World!", result);
	}

	@Test
	public void invokeHelloStrangerWithValidUser() throws Exception {
		Map<String, Object> reqContext = ((BindingProvider) proxy)
				.getRequestContext();
		reqContext.put(BindingProvider.USERNAME_PROPERTY, "user");
		reqContext.put(BindingProvider.PASSWORD_PROPERTY, "abhijitsarkar");

		final String result = proxy.helloStranger("World");
		Assert.assertEquals("Hello World!", result);
	}

	/* Tests for helloAuthenticated method */

	@Test
	public void invokeHelloAuthenticatedWithoutUsernameAndPassord()
			throws Exception {
		try {
			proxy.helloAuthenticated("World");
			Assert.fail("Test should fail, HTTP response '401: Unauthorized' was expected");
		} catch (WebServiceException e) {
			// failure is expected
			Assert.assertTrue("HTTPException '401: Unauthorized' was expected",
					e.getMessage().contains("401: Unauthorized"));
		}
	}

	@Test
	public void invokeHelloAuthenticatedWithBadPassword() throws Exception {
		Map<String, Object> reqContext = ((BindingProvider) proxy)
				.getRequestContext();
		reqContext.put(BindingProvider.USERNAME_PROPERTY, "user");
		reqContext.put(BindingProvider.PASSWORD_PROPERTY, "badPassword");

		try {
			proxy.helloAuthenticated("World");
			Assert.fail("Test should fail, HTTP response '401: Unauthorized' was expected");
		} catch (WebServiceException e) {
			// failure is expected
			Assert.assertTrue("HTTPException '401: Unauthorized' was expected",
					e.getMessage().contains("401: Unauthorized"));
		}
	}

	@Test
	public void invokeHelloAuthenticatedWithUnauthorizedUser() throws Exception {
		Map<String, Object> reqContext = ((BindingProvider) proxy)
				.getRequestContext();
		reqContext.put(BindingProvider.USERNAME_PROPERTY, "unauthorizedUser");
		reqContext.put(BindingProvider.PASSWORD_PROPERTY, "doesNotMatter");

		try {
			proxy.helloAuthenticated("World");
			Assert.fail("Test should fail, user shouldn't be allowed to invoke that method");
		} catch (WebServiceException e) {
			// failure is expected
			Assert.assertTrue("HTTPException '401: Unauthorized' was expected",
					e.getMessage().contains("401: Unauthorized"));
		}
	}

	@Test
	public void invokeHelloAuthenticatedWithValidUser() throws Exception {
		Map<String, Object> reqContext = ((BindingProvider) proxy)
				.getRequestContext();
		reqContext.put(BindingProvider.USERNAME_PROPERTY, "user");
		reqContext.put(BindingProvider.PASSWORD_PROPERTY, "abhijitsarkar");

		final String result = proxy.helloAuthenticated("World");
		Assert.assertEquals("Hello World!", result);
	}

	/* Tests for helloAuthorized method */

	@Test
	public void invokeHelloAuthorizedWithoutUsernameAndPassord()
			throws Exception {
		try {
			proxy.helloAuthorized("World");
			Assert.fail("Test should fail, HTTP response '401: Unauthorized' was expected");
		} catch (WebServiceException e) {
			// failure is expected
			Assert.assertTrue("HTTPException '401: Unauthorized' was expected",
					e.getMessage().contains("401: Unauthorized"));
		}
	}

	@Test
	public void invokeHelloAuthorizedWithBadPassword() throws Exception {
		Map<String, Object> reqContext = ((BindingProvider) proxy)
				.getRequestContext();
		reqContext.put(BindingProvider.USERNAME_PROPERTY, "user");
		reqContext.put(BindingProvider.PASSWORD_PROPERTY, "badPassword");

		try {
			proxy.helloAuthorized("World");
			Assert.fail("Test should fail, HTTP response '401: Unauthorized' was expected");
		} catch (WebServiceException e) {
			// failure is expected
			Assert.assertTrue("HTTPException '401: Unauthorized' was expected",
					e.getMessage().contains("401: Unauthorized"));
		}
	}

	@Test
	public void invokeHelloAuthorizedWithUnauthorizedUser() throws Exception {
		Map<String, Object> reqContext = ((BindingProvider) proxy)
				.getRequestContext();
		reqContext.put(BindingProvider.USERNAME_PROPERTY, "unauthorizedUser");
		reqContext.put(BindingProvider.PASSWORD_PROPERTY, "doesNotMatter");

		try {
			proxy.helloAuthorized("World");
			Assert.fail("Test should fail, user shouldn't be allowed to invoke that method");
		} catch (WebServiceException e) {
			// failure is expected
			Assert.assertTrue("HTTPException '401: Unauthorized' was expected",
					e.getMessage().contains("401: Unauthorized"));
		}
	}

	@Test
	public void invokeHelloAuthorizedWithInvalidRole() throws Exception {
		Map<String, Object> reqContext = ((BindingProvider) proxy)
				.getRequestContext();
		reqContext.put(BindingProvider.USERNAME_PROPERTY, "guest");
		reqContext.put(BindingProvider.PASSWORD_PROPERTY, "abhijitsarkar");

		try {
			proxy.helloAuthorized("World");
			Assert.fail("Test should fail, user shouldn't be allowed to invoke that method");
		} catch (WebServiceException e) {
			// failure is expected
			Assert.assertTrue("HTTPException '403: Forbidden' was expected", e
					.getMessage().contains("403: Forbidden"));
		}
	}

	@Test
	public void invokeHelloAuthorizedWithValidRole() throws Exception {
		Map<String, Object> reqContext = ((BindingProvider) proxy)
				.getRequestContext();
		reqContext.put(BindingProvider.USERNAME_PROPERTY, "user");
		reqContext.put(BindingProvider.PASSWORD_PROPERTY, "abhijitsarkar");

		final String result = proxy.helloAuthorized("World");
		Assert.assertEquals("Hello World!", result);
	}
}